from django.contrib import admin

from .models import BackendClient, BackendPayment, BackendPlan, BackendSubscription, BackendEvent
from . import get_subscription_model, get_plan_model

Subscription = get_subscription_model()
Plan = get_plan_model()


class PlanAdmin(admin.ModelAdmin):
    list_display = ('name', 'amount', 'trial_period_days', 'visible')
    list_filter = ('visible', )
    search_fields = ('name', 'amount')


class SubscriptionAdmin(admin.ModelAdmin):
    list_display = ('plan', 'quantity', 'total', 'currency', 'deleted')
    list_filter = ('plan', 'quantity', 'currency', 'deleted')


admin.site.register(Plan, PlanAdmin)
admin.site.register(Subscription, SubscriptionAdmin)


class BackendClientAdmin(admin.ModelAdmin):
    list_display = ('backend', 'created_at', 'email', 'description')
    list_filter = ('backend', )


class BackendPaymentAdmin(admin.ModelAdmin):
    list_display = ('backend', 'created_at', 'payment_type', 'card_type', 'holder')
    list_filter = ('backend', 'payment_type', 'card_type')


class BackendPlanAdmin(admin.ModelAdmin):
    list_display = ('backend', 'created_at', 'plan', 'quantity', 'subscription_count_active', 'subscription_count_inactive')
    list_filter = ('backend', )


class BackendSubscriptionAdmin(admin.ModelAdmin):
    list_display = ('backend', 'created_at', 'status')
    list_filter = ('backend', )


class BackendEventAdmin(admin.ModelAdmin):
    list_display = ('backend', 'created_at', 'event_type', 'processed')
    list_filter = ('backend', 'processed', 'event_type')

admin.site.register(BackendClient, BackendClientAdmin)
admin.site.register(BackendPayment, BackendPaymentAdmin)
admin.site.register(BackendPlan, BackendPlanAdmin)
admin.site.register(BackendSubscription, BackendSubscriptionAdmin)
admin.site.register(BackendEvent, BackendEventAdmin)



"""
JSONField automatically serializes most Python terms to JSON data.
Creates a TEXT field with a default value of "{}".  See test_json.py for
more information.

 from django.db import models
 from django_extensions.db.fields import json

 class LOL(models.Model):
     extra = json.JSONField()

from "django-extensions"
https://github.com/django-extensions/django-extensions/blob/master/django_extensions/db/fields/json.py

contains fix to allow native postgres JSON field:

 def db_type(self, connection):
     return 'json'
"""

import six
from decimal import Decimal
from django.db import models
from django.conf import settings
from django.core.serializers.json import DjangoJSONEncoder

try:
    # Django <= 1.6 backwards compatibility
    from django.utils import simplejson as json
except ImportError:
    # Django >= 1.7
    import json


def dumps(value):
    return DjangoJSONEncoder().encode(value)


def loads(txt):
    value = json.loads(
        txt,
        parse_float=Decimal,
        encoding=settings.DEFAULT_CHARSET
    )
    return value


class JSONDict(dict):
    """
    Hack so repr() called by dumpdata will output JSON instead of
    Python formatted data.  This way fixtures will work!
    """
    def __repr__(self):
        return dumps(self)


class JSONList(list):
    """
    As above
    """
    def __repr__(self):
        return dumps(self)


class JSONField(six.with_metaclass(models.SubfieldBase, models.TextField)):
    """JSONField is a generic textfield that neatly serializes/unserializes
    JSON objects seamlessly.  Main thingy must be a dict object."""

    def __init__(self, *args, **kwargs):
        default = kwargs.get('default', None)
        if default is None:
            kwargs['default'] = '{}'
        elif isinstance(default, (list, dict)):
            kwargs['default'] = dumps(default)
        models.TextField.__init__(self, *args, **kwargs)

    def to_python(self, value):
        """Convert our string value to JSON after we load it from the DB"""
        if value is None or value == '':
            return {}
        elif isinstance(value, six.string_types):
            res = loads(value)
            if isinstance(res, dict):
                return JSONDict(**res)
            else:
                return JSONList(res)

        else:
            return value

    def get_db_prep_save(self, value, connection):
        """Convert our JSON object to a string before we save"""
        if not isinstance(value, (list, dict)):
            return super(JSONField, self).get_db_prep_save("", connection=connection)
        else:
            return super(JSONField, self).get_db_prep_save(dumps(value),
                                                           connection=connection)

    def south_field_triple(self):
        "Returns a suitable description of this field for South."
        # We'll just introspect the _actual_ field.
        from south.modelsinspector import introspector
        field_class = "django.db.models.fields.TextField"
        args, kwargs = introspector(self)
        # That's our definition!
        return (field_class, args, kwargs)

    def db_type(self, connection):
        if connection.vendor == 'postgresql' and connection.pg_version > 90200:
            return 'json'
        else:
            return super(JSONField, self).db_type(connection)



# -*- coding: UTF-8 -*-

import factory
import random
from factory import fuzzy

from .models import AbstractSubscription, AbstractPlan, PLAN_INTERVAL_CHOICES

from . import get_plan_model, get_subscription_model

INTERVAL_CHOICES = [x for (x, y) in PLAN_INTERVAL_CHOICES]


class AbstractPlanFactory(factory.django.DjangoModelFactory):
    ABSTRACT_FACTORY = True
    FACTORY_FOR = AbstractPlan

    name = factory.LazyAttribute(lambda o: "{} {}{}".format(
        random.choice(['Simple', 'Almost Free', 'Wow so', 'Lean', 'Product']),
        random.choice(['Premium', 'Pro', 'Plus', 'Extended', 'Enterprise']),
        random.choice(['', ' 50% off', ' Extended', ' X', ' 9000']),
    ))
    amount = fuzzy.FuzzyInteger(1, 50000)
    interval = fuzzy.FuzzyChoice(INTERVAL_CHOICES)
    interval_count = fuzzy.FuzzyInteger(1, 3)
    trial_period_days = fuzzy.FuzzyChoice([None, 7, 30])
    visible = True


class PlanFactory(AbstractPlanFactory):
    FACTORY_FOR = get_plan_model()


class AbstractSubscriptionFactory(factory.django.DjangoModelFactory):
    ABSTRACT_FACTORY = True
    FACTORY_FOR = AbstractSubscription

    quantity = fuzzy.FuzzyInteger(1, 100)
    currency = 'EUR'
    deleted = False


class SubscriptionFactory(AbstractSubscriptionFactory):
    FACTORY_FOR = get_subscription_model()

    plan = factory.SubFactory(PlanFactory)



from django.dispatch import Signal

new_subscription = Signal(providing_args=['subscription'])
new_subscription.__doc__ = """Sent after creating a subscription"""

payment_error = Signal(providing_args=['error_code', 'exception'])
payment_error.__doc__ = """Is sent whenever an error occurs during payment"""



from django.views.generic import TemplateView
from django.views.generic.base import RedirectView


from . import get_plan_model
from .utils import get_default_backend


Plan = get_plan_model()


class PaymentsContextMixin(object):

    def get_context_data(self, **kwargs):
        context = super(PaymentsContextMixin, self).get_context_data(**kwargs)
        context.update({
            "PLAN_CHOICES": Plan.objects.all(),
            "BACKEND": get_default_backend()
        })
        return context


class SubscribeView(RedirectView):

    permanent = False

    def get_redirect_url(self, *args, **kwargs):
        backend = get_default_backend()
        url, _, _ = backend.PaymentProcessor.get_gateway_url(None)
        return url

# class SubscribeView(PaymentsContextMixin, TemplateView):
#     template_name = "subscribe.html"

#     def get_context_data(self, **kwargs):
#         context = super(SubscribeView, self).get_context_data(**kwargs)
#         context.update({
#             "organization": "ubergrape",  # todo
#             "form": PlanForm
#         })
#         return context


class ChangeCardView(PaymentsContextMixin, TemplateView):
    template_name = "change_card.html"


class CancelView(PaymentsContextMixin, TemplateView):
    template_name = "cancel.html"


class ChangePlanView(SubscribeView):
    template_name = "change_plan.html"


class HistoryView(PaymentsContextMixin, TemplateView):
    template_name = "history.html"


class SubscriptionSuccessView(TemplateView):
    template_name = "subscription_success.html"


class SubscriptionFailureView(TemplateView):
    template_name = "subscription_failure.html"



from django.utils.translation import ugettext_lazy as _

from .utils import get_backend_choices


BACKEND_CHOICES = get_backend_choices()

SUBSCRIPTION_STATUS_CHOICES = (
    ('new', _('new')),
    ('in_progress', _('in progress')),
    ('partially_paid', _('partially paid')),
    ('paid', _('paid')),
    ('failed', _('failed')),
)

TRANSACTION_STATUS_CHOICES = (
    ('open', _('open')),
    ('pending', _('pending')),
    ('closed', _('closed')),
    ('failed', _('failed')),
    ('partial_refunded', _('partial_refunded')),
    ('refunded', _('refunded')),
    ('preauthorize', _('preauthorize')),
    ('chargeback', _('chargeback')),
)


PLAN_INTERVAL_CHOICES = (
    ('year', _('year')),
    ('month', _('month')),
    ('week', _('week')),
    ('day', _('day')),
)

PLAN_TYPE_CHOICES = (
    ('fixed', _('fixed')),
    ('per_user', _('per_user')),
)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


setup(
    name='django-abo',
    version='0.1.3',
    description='Recurring payment / subscription handling for Django, supporting different payment gateways',
    url='https://github.com/ubergrape/django-abo',
    author='Stefan Kröner',
    author_email='sk@ubergrape.com',
    license='BSD',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    zip_safe=False,
    platforms=["any"],
    install_requires=[
        'django >= 1.6',
        'pymill',
        'requests'
    ],
    tests_require=[
        'factory-boy >= 2.3.0',
        'requests'
    ]
)
